<?php
namespace Elementor;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Jet_Elements_Contact_Form7 extends Jet_Elements_Base {

	public function get_name() {
		return 'contact-form-7';
	}

	public function get_title() {
		return esc_html__( 'Contact Form 7', 'jet-elements' );
	}

	public function get_icon() {
		return 'jetelements-icon-14';
	}

	public function get_categories() {
		return array( 'cherry' );
	}

	protected function _register_controls() {
		$this->start_controls_section(
			'section_settings',
			array(
				'label' => esc_html__( 'Form', 'jet-elements' ),
			)
		);

		$this->add_control( 'form_shortcode', array(
			'label'   => esc_html__( 'Columns', 'jet-elements' ),
			'type'    => Controls_Manager::SELECT,
			'default' => '',
			'options' => $this->get_availbale_forms(),
		) );

		$this->end_controls_section();
	}

	/**
	 * Retrieve available forms list.
	 * @return [type] [description]
	 */
	protected function get_availbale_forms() {

		if ( ! class_exists( 'WPCF7_ContactForm' ) ) {
			return array();
		}

		$forms = \WPCF7_ContactForm::find( array(
			'orderby' => 'title',
			'order'   => 'ASC',
		) );

		if ( empty( $forms ) ) {
			return array();
		}

		$result = array();

		foreach ( $forms as $item ) {
			$key        = sprintf( '%1$s::%2$s', $item->id(), $item->title() );
			$result[ $key ] = $item->title();
		}

		return $result;
	}

	protected function render() {

		$settings = $this->get_settings();

		$this->__context = 'render';

		$this->__open_wrap();

		$shortcode = $this->get_settings( 'form_shortcode' );
		$data      = explode( '::', $shortcode );

		if ( ! empty( $data ) && 2 === count( $data ) ) {
			echo do_shortcode( sprintf( '[contact-form-7 id="%1$d" title="%2$s"]', $data[0], $data[1] ) );
		}

		$this->__close_wrap();

	}

}
